# architecture overview

## components:

- ✅ ComputerKeyboard component
  - handles azerty/qwerty input
  - ouputs GM data to child-as-a-function

- ✅ AudioOutput component
  - finally sends the audio signal to a master summing component after it has been generated by an <Instrument> and processed by any <Effect>

- ✅ makeSynth HOC
  - provides a midiMsg prop & handles monophonic triggers.
  - provides output prop and sets up connections accordingly.
  - params: options (for now this is: `{ synth }`), Component.

- ✅ makeEffect HOC
  - provides input & output props and sets up connections accordingly.
  - params: options (for now this is: `{ effectNode }`), Component.


---
still todo:

- get rid of TONE dependency in favor of own framework (or explicit use of web audio api);

- MidiConnector component
  - handles midi input
  - ouputs GM data to child-as-a-function

- withMidiOutput HOC
  - receives GM from any component
  - outputs GM through the web-midi-api to be used in any DAW.
  - possibly use https://www.npmjs.com/package/webmidi under the hood

- StepSequencer component
  - handles step input (either through user clicks, OR through midi signals from the midi input component)
  - outputs interval stream of GM data.

- Sampler component
  - takes midi input
  - generates actual sound from the sample
  - renders a visual

- Synth components
  - the basics are there
  - need controls
  - needs API to render your OWN visualisation & synth controls
    - so this could be done through a renderer prop, which receives all the necessary params (which is basically `this.synth` -- right...?)

- Other ideas:
  - matrix sequencer
  - 808, 707, 606 emulations
  - synthesisers
  - effects (manip audio)
  - arpeggiators & other midi effects
  - ...


## Connector API

This is what the current API looks like, with a Connector context providing component.
Input and output props are basically IDs, and internally everything gets routed automatically according to these IDs.

<Connector> provides two context types:
- `setOutput` -- a function that, when called with a `synth` (or basically, audionode) and a `destination` (an ID on which to save the connection), saves the audionode to a key on a `connections` object.

- `connections` -- an object containing all audionodes. The `makeSynth` and `makeEffect` HOCs, and the `AudioOutput` component all make use of this object to connect with any audionode based on the `input` prop.


## example of the current API:
```js
import React, { Component } from "react";
import { ComputerKeyboard, MonoSynth, AudioOutput, Distortion, Tremolo, Connector } from "react-audio-tools";

class App extends Component {
  state = {}

  constructor() {
    super();
    this.state = {
      input: "tremolo",
    };
  }

  render() {
    return (
      <Connector>
        it is possible to route the inputs and outputs however you like:
        <button onClick={() => this.setState({ input: "tremolo" })}>{"synth -> tremolo -> output"}</button>
        <button onClick={() => this.setState({ input: "distortion" })}>{"synth -> distortion -> output"}</button>

        <ComputerKeyboard controlled keyboardLayout="azerty">
          {msg => <MonoSynth midiMsg={msg} output={"monosynth"} />}
        </ComputerKeyboard>

        <Distortion input={"monosynth"} output={"distortion"} />
        <Tremolo input={"monosynth"} output={"tremolo"} />
        <AudioOutput input={this.state.input} />
      </Connector>
    );
  }
}

export default App;


```